// --------------------------------------------------------------------------
// External files inclusions.
// --------------------------------------------------------------------------
#include <pic.h>                                                // Use a PIC16F876A.
#include "io.h"                                                 // I/O lines definitions.
#include "delay.h"                                              // Delay procedure



// --------------------------------------------------------------------------
// This procedure will read the specified ADC channel and return the 10bit
// result. Make sure ADCON1 (PCFGx) is correctly set in IO_init().
// --------------------------------------------------------------------------
unsigned int adc_read(unsigned char channel)
{
    unsigned int result;

//    PCFG0 = 0;                                                  // Configure RA0, RA1 and RA3 as analog inputs (rest is digital).
//    PCFG1 = 0;                                                  // Even if this has already been set by IO_init it is
//    PCFG2 = 1;                                                  // repeated here, just to make sure.
//    PCFG3 = 0;

    ADFM = 0;                                                   // Result is left justified.

    ADCON0 = (channel & 0x07) << 3;                             // Select correct channel.

    ADCS0 = 0;                                                  // Tad = 32*Tosc. Works up to 20MHz xtals.
    ADCS1 = 1;

    ADON = 1;                                                   // Switch ADC on.

    delay(1);                                                   // Wait at least 20us to allow correct sampling.

    ADGO = 1;                                                   // Start A/D conversion.
    while (ADGO == 1) ;                                         // Wait untill conversion is done.

    result = ((unsigned int)ADRESH << 2) | ((unsigned int)ADRESL >> 6);   // Get ADC result.
    ADON = 0;                                                   // Switch ADC off.

    return result;
}
